{% extends "registration/registration_base.html" %}
{% load i18n %}

{% block title %}{% trans "Approval Failure" %}{% endblock %}

{% block content %}
<p>{% trans "Account Approval failed." %}</p>
{% endblock %}


{% comment %}
**registration/admin_approve.html**

Used if account activation fails. With the default setup, has the following context:

``activation_key``
    The activation key used during the activation attempt.
{% endcomment %}
