{% extends "registration/registration_base.html" %}
{% load crispy_forms_tags %}
{% load i18n %}

{% block title %}{% trans "Log in" %}{% endblock %}

{% block content %}
<form method="post" action="" class="row g-3 needs-validation" novalidate>
    {% csrf_token %}
    {{ form|crispy }}
    <input type="submit" class="btn btn-sm btn-outline-primary" value="{% trans 'Log in' %}" />
    <input type="hidden" name="next" value="{{ next }}" />
</form>

<p class="float-start">{% trans "Forgot your password?" %} <a href="{% url 'auth_password_reset' %}">{% trans "Reset it" %}</a>.</p>
<p class="float-end">{% trans "Not a member?" %} <a href="{% url 'registration_register' %}">{% trans "Register" %}</a>.</p>
{% endblock %}


{% comment %}
**registration/login.html**

It's your responsibility to provide the login form in a template called
registration/login.html by default. This template gets passed four
template context variables:

``form``
    A Form object representing the login form. See the forms
    documentation for more on Form objects.

``next``
    The URL to redirect to after successful login. This may contain a
    query string, too.

``site``
    The current Site, according to the SITE_ID setting. If you don't
    have the site framework installed, this will be set to an instance
    of RequestSite, which derives the site name and domain from the
    current HttpRequest.

``site_name``
    An alias for site.name. If you don't have the site framework
    installed, this will be set to the value of
    request.META['SERVER_NAME']. For more on sites, see The
    "sites" framework.
{% endcomment %}
