{% extends "registration/registration_base.html" %}
{% load i18n %}

{% block title %}{% trans "Logged out" %}{% endblock %}

{% block content %}
<p>{% trans "Successfully logged out" %}.</p>
{% endblock %}
