{% extends "registration/registration_base.html" %}
{% load crispy_forms_tags %}
{% load i18n %}

{% block title %}{% trans "Change password" %}{% endblock %}

{% block content %}
<form method="post" action="" class="row g-3 needs-validation" novalidate>
    {% csrf_token %}
    {{ form|crispy }}
    <input type="submit" class="btn btn-sm btn-outline-primary" value="{% trans 'Change password' %}" />
</form>
{% endblock %}


{# This is used by django.contrib.auth #}
